//--------------------------------------------------------------
// File     : main.c
// Datum    : 17.06.2015
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// Board    : STM32F429-Board von UB (Version 1.0)
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : CMSIS_BOOT, M4_CMSIS_CORE
// Funktion : Hauptprogramm
// Hinweis  : Diese zwei Files muessen auf 8MHz stehen
//              "cmsis_boot/stm32f4xx.h"
//              "cmsis_boot/system_stm32f4xx.c"
// In Configuration diese Define hinzufgen :
// "STM32F429_439xx" , "__ASSEMBLY__" , "USE_STDPERIPH_DRIVER"
//--------------------------------------------------------------
// Beschreibung :
// LoLevel-Test vom eigenbau STM32F429-Board von UB (Version 1.0)
//
// Check vom TFT-Display
// Testbild wird angezeigt
//
// Statusausgabe per COM-6 (3,3V UART) 115200 Baud
//--------------------------------------------------------------

#include "main.h"
#include "stm32_ub_systick.h"
#include "stm32_ub_uart.h"
#include <stdio.h>
#include "stm32_ub_lcd_480x272.h"


//----------------------------------------------------------------------
// gefuelltes Rechteck zeichnen
// (im RGB-Farbraum)
//----------------------------------------------------------------------
void fillrect(uint16_t xp, uint16_t yp, uint16_t w, uint16_t h, uint16_t c)
{
  uint16_t x,y;

  for(y=0;y<h;y++) {
    for(x=0;x<w;x++) {
      UB_LCD_SetCursor2Draw(xp+x,yp+y);
      UB_LCD_DrawPixel(c);
    }
  }
}

//----------------------------------------------------------------------
// gefuelltes Rechteck zeichnen
// (im HSV-Farbraum
//----------------------------------------------------------------------
void fillrect_hsv(uint16_t xp, uint16_t yp, uint16_t w, uint16_t l, uint16_t h, uint16_t s, uint16_t v)
{
  uint8_t r,g,b,diff;
  uint16_t c;

  // Grenzwerte
  if(h>359) h=359;
  if(s>100) s=100;
  if(v>100) v=100;

  if(h < 61) {
    r = 255;
    g = (425 * h) / 100;
    b = 0;
  }else if(h < 121){
    r = 255 - ((425 * (h-60))/100);
    g = 255;
    b = 0;
  }else if(h < 181){
    r = 0;
    g = 255;
    b = (425 * (h-120))/100;
  }else if(h < 241){
    r = 0;
    g = 255 - ((425 * (h-180))/100);
    b = 255;
  }else if(h < 301){
    r = (425 * (h-240))/100;
    g = 0;
    b = 255;
  }else {
    r = 255;
    g = 0;
    b = 255 - ((425 * (h-300))/100);
  }

  s = 100 - s;
  diff = ((255 - r) * s)/100;
  r = r + diff;
  diff = ((255 - g) * s)/100;
  g = g + diff;
  diff = ((255 - b) * s)/100;
  b = b + diff;

  r = ((r * v)/100)>>3;
  g = ((g * v)/100)>>2;
  b = ((b * v)/100)>>3;

  c=b+(g<<5)+(r<<11);
  fillrect(xp,yp,w,l,c);
}


//----------------------------------------------------------------------
// Testscreen anzeigen (480 x 272)
//----------------------------------------------------------------------
void testscreen(void)
{
  uint16_t n,xp;

  // rainbow colors
  for(n=0;n<360;n++) {
    fillrect_hsv(n+1,1,1,48,n,100,100);
  }
  // Farbverlauf in rot
  xp=1;
  for(n=0;n<100;n++) {
    fillrect_hsv(xp,50,4,48,0,n,100);
    xp+=4;
  }
  // Farbverlauf in gruen
  xp=1;
  for(n=0;n<100;n++) {
    fillrect_hsv(xp,100,4,48,120,n,100);
    xp+=4;
  }
  // Farbverlauf in blau
  xp=1;
  for(n=0;n<100;n++) {
    fillrect_hsv(xp,150,4,48,240,n,100);
    xp+=4;
  }
  // Rechtecke mit Farben
  fillrect(1,200,48,48,RGB_COL_BLACK);     // schwarz
  fillrect(50,200,48,48,RGB_COL_BLUE);     // blau
  fillrect(100,200,48,48,RGB_COL_GREEN);   // gruen
  fillrect(150,200,48,48,RGB_COL_RED);     // rot
  fillrect(200,200,48,48,RGB_COL_WHITE);   // weiss
  fillrect(250,200,48,48,RGB_COL_CYAN);    // cyan
  fillrect(300,200,48,48,RGB_COL_MAGENTA); // magenta
  fillrect(350,200,48,48,RGB_COL_YELLOW);  // gelb
  fillrect(400,200,48,48,0xD69A);          // grau
}




//----------------------------------------------------------------------
int main(void)
{

  SystemInit(); // Quarz Einstellungen aktivieren

  // init vom sytsick
  UB_Systick_Init();

  // init der uart
  UB_Uart_Init();

  UB_Systick_Pause_ms(500);

  // powerOn string
  UB_Uart_SendString(COM6,"F429-Board",CRLF);
  UB_Uart_SendString(COM6,"TFT-LCD-Check",CRLF);
  UB_Uart_SendString(COM6," ",CRLF);
  UB_Systick_Pause_ms(500);

  // init vom LCD
  UB_LCD_Init();
  // auf FullScreen
  UB_LCD_LayerInit_Fullscreen();

  // auf Layer-2 schalten
  UB_LCD_SetLayer_2();
  // mit gruen fuellen
  UB_LCD_FillLayer(RGB_COL_WHITE);

  // show Testscreen
  testscreen();

  UB_Uart_SendString(COM6,"ready",CRLF);


  while(1) {

  }
}

